@@ -20,7 +20,7 @@
 
 #include "application.h"
 #include "dialog_new_database.h"
-#include "dialog_progress_creating.h"
+#include  <glom/libglom/dialog_progress_creating.h>
 #include "translation/dialog_change_language.h"
 #include "translation/window_translations.h"
 #include <glom/libglom/utils.h>
@@ -405,6 +405,26 @@ void App_Glom::on_menu_userlevel_operator()
     m_pFrame->on_menu_userlevel_Operator(m_action_menu_userlevel_operator);
 }
 
+void App_Glom::on_menu_file_close() //override
+{
+  // Call the base class implementation:
+  Bakery::App_WithDoc_Gtk::on_menu_file_close();
+}
+
+
+void App_Glom::stop_self_hosting_of_document_database()
+{
+  Document_Glom* pDocument = static_cast<Document_Glom*>(get_document());
+  if(pDocument && pDocument->get_connection_is_self_hosted())
+  {
+    ConnectionPool* connection_pool = ConnectionPool::get_instance();
+    if(!connection_pool)
+      return;
+
+    connection_pool->stop_self_hosting();
+  }
+}
+
 Bakery::App* App_Glom::new_instance() //Override
 {
   Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_main");
@@ -440,7 +460,9 @@ bool App_Glom::on_document_load()
   //m_pFrame->load_from_document();
   Document_Glom* pDocument = static_cast<Document_Glom*>(get_document());
   if(!pDocument)
+  {
     return false;
+  }
   else
   {
     //Connect signals:
@@ -509,6 +531,17 @@ bool App_Glom::on_document_load()
       {
         //Set the connection details in the ConnectionPool singleton.
         //The ConnectionPool will now use these every time it tries to connect.
+        if(pDocument->get_connection_is_self_hosted())
+        {
+          // TODO: sleep, to give postgres time to start?
+          connection_pool->set_self_hosted(pDocument->get_connection_self_hosted_directory_uri());
+          connection_pool->start_self_hosting(); //Stopped in on_menu_file_close().
+        }
+        else
+        {
+          connection_pool->set_self_hosted(std::string());
+        }
+
         connection_pool->set_host(pDocument->get_connection_server());
         connection_pool->set_user(pDocument->get_connection_user());
         connection_pool->set_database(pDocument->get_connection_database());
@@ -520,12 +553,34 @@ bool App_Glom::on_document_load()
         {
           bool test = false;
           if(is_example)
+          {
+            //Allow the user to change the database title, 
+            //and choose whether the new database will be self-hosting.
+            Glib::ustring db_title = pDocument->get_database_title();
+            bool self_hosted = false;
+
+            const bool not_cancelled = show_dialog_new_database(this, db_title, self_hosted);   
+            if(!not_cancelled)
+            {
+              return false;
+            }
+    
+            pDocument->set_database_title(db_title);
+            pDocument->set_connection_is_self_hosted(self_hosted);
+
             test = m_pFrame->connection_request_password_and_choose_new_database_name();
+          }
           else
             test = m_pFrame->connection_request_password_and_attempt();
 
           if(!test) //It usually throws an exception instead of returning false.
+          {
+            //Stop self-hosting, if we are doing that:
+            std::cout << "debug: calling stop_self_hosting_of_document_database()" << std::endl;
+            stop_self_hosting_of_document_database();
+
             return false; //Failed. Close the document.
+          }
 
           if(is_example)
           {
@@ -678,7 +733,6 @@ bool App_Glom::offer_new_or_existing()
     if(!file_uri.empty())
       open_document(file_uri);
 
-
     //Check that a document was opened:
     Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
     if(document->get_file_uri().empty())
@@ -701,90 +755,79 @@ bool App_Glom::offer_new_or_existing()
 
       //Each new document must have an associated new database,
       //so ask the user for the name of one to create:
-      Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_new_database");
-      if(refXml)
+      
+
+      //Set suitable defaults:
+      const Glib::ustring filename = document->get_name(); //Get the filename without the path and extension.
+      Glib::ustring db_title = Utils::title_from_string( filename ); //Start with something suitable.
+      bool self_hosted = false;
+
+      bool keep_asking = true;
+      while(keep_asking)
       {
-        Dialog_NewDatabase* dialog = 0;
-        refXml->get_widget_derived("dialog_new_database", dialog);
-        if(dialog)
+        const bool not_cancelled = show_dialog_new_database(this, db_title, self_hosted);
+            
+        if(not_cancelled)
         {
-          std::auto_ptr<Dialog_NewDatabase> dialog_owner(dialog); //This will delete the dialog even when we return in the middle of this function.
-          dialog->set_transient_for(*this);
+          //Create a database name based on the title.
+          //The user will (almost) never see this anyway but it's nicer than using a random number:
+          Glib::ustring db_name = Utils::create_name_from_title(db_title);
 
-          //Set suitable defaults:
-          const Glib::ustring filename = document->get_name(); //Get the filename without the path and extension.
-          dialog->set_input( Utils::title_from_string( filename ) ); //Start with something suitable.
+          //Prefix glom_ to the database name, so it's more obvious
+          //for the system administrator.
+          //This database name should never be user-visible again, either prefixed or not prefixed.
+          db_name = "glom_" + db_name;
 
-          bool keep_asking = true;
-          while(keep_asking)
+          if(!db_name.empty()) //The dialog prevents this anyway.
           {
-            int response = Glom::Utils::dialog_run_with_help(dialog, "dialog_new_database");
+            //Connect to the server and choose a variation of this db_name that does not exist yet:
+            document->set_connection_database(db_name);
+            document->set_connection_is_self_hosted(self_hosted);
+               
+            const bool connected = m_pFrame->connection_request_password_and_choose_new_database_name();
+            if(!connected)
+              return false;
 
-            if(response == Gtk::RESPONSE_OK)
+            const bool db_created = m_pFrame->create_database(document->get_connection_database(), db_title, false /* do not request password */);
+            if(db_created)
             {
-              Glib::ustring db_title;
-              dialog->get_input(db_title);
+              keep_asking = false;
 
-              //Create a database name based on the title.
-              //The user will (almost) never see this anyway but it's nicer than using a random number:
-              Glib::ustring db_name = Utils::create_name_from_title(db_title);
+              //document->set_connection_database(db_name); //Select the database that was just created.
 
-              //Prefix glom_ to the database name, so it's more obvious
-              //for the system administrator.
-              //This database name should never be user-visible again, either prefixed or not prefixed.
-              db_name = "glom_" + db_name;
-
-              if(!db_name.empty()) //The dialog prevents this anyway.
+              /*
+              ConnectionPool* connection_pool = ConnectionPool::get_instance();
+              if(connection_pool)
               {
-                //Connect to the server and choose a variation of this db_name that does not exist yet:
-                document->set_connection_database(db_name);
-                const bool connected = m_pFrame->connection_request_password_and_choose_new_database_name();
-                if(!connected)
-                  return false;
-
-                const bool db_created = m_pFrame->create_database(document->get_connection_database(), db_title, false /* do not request password */);
-                if(db_created)
-                {
-                  keep_asking = false;
-
-                  //document->set_connection_database(db_name); //Select the database that was just created.
-
-                  /*
-                  ConnectionPool* connection_pool = ConnectionPool::get_instance();
-                  if(connection_pool)
-                  {
-                    connection_pool->set_database(db_name); //The rest has been set while creating the database.
-                  }
-                  */
-
-                  const Glib::ustring database_name_used = document->get_connection_database();
-                  ConnectionPool::get_instance()->set_database(database_name_used);
-                  document->set_database_title(db_title);
-                  m_pFrame->set_databases_selected(database_name_used);
-                }
-                else
-                {
-                  //Ask again:
-                  return offer_new_or_existing();
-                }
+                connection_pool->set_database(db_name); //The rest has been set while creating the database.
               }
-              else
-              {
-                g_warning(" App_Glom::offer_new_or_existing(): db_name is empty.");
-                //And ask again, by going back to the start of the while() loop.
-              }
-            } /* if(response) */
+              */
+
+              const Glib::ustring database_name_used = document->get_connection_database();
+              ConnectionPool::get_instance()->set_database(database_name_used);
+              document->set_database_title(db_title);
+              m_pFrame->set_databases_selected(database_name_used);
+            }
             else
             {
-              return false; //The user cancelled.
+              //Ask again:
+              return offer_new_or_existing();
             }
-          } /* while() */
-
-          return true; //File successfully created.
+          }
+          else
+          {
+             g_warning(" App_Glom::offer_new_or_existing(): db_name is empty.");
+             //And ask again, by going back to the start of the while() loop.
+          }
+        } /* if(response) */
+        else
+        {
+          return false; //The user cancelled.
         }
-       }
+      } /* while() */
+
+      return true; //File successfully created.
 
-      return false; //Creation of new document failed.
     }
     else
     {
@@ -884,14 +927,15 @@ bool App_Glom::recreate_database(bool& user_cancelled)
   //Show the user that something is happening, because the INSERTS might take time.
   //TOOD: This doesn't actually show up until near the end, even with Gtk::Main::instance()->iteration().
   std::auto_ptr<Dialog_ProgressCreating> dialog_progress;
-  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_progress_creating_from_example");
+  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_progress");
   if(refXml)
   {
     Dialog_ProgressCreating* dialog_progress_temp = 0;
-    refXml->get_widget_derived("window_progress_creating_from_example", dialog_progress_temp);
+    refXml->get_widget_derived("window_progress", dialog_progress_temp);
     if(dialog_progress_temp)
     {
-      dialog_progress.reset(dialog_progress_temp); //The dialog will be deleted (and hidden) when this function returns.
+      dialog_progress_temp->set_message(_("Creating Glom Database"), _("Creating Glom database from example file."));
+      dialog_progress.reset(dialog_progress_temp); //Put the dialog in an auto_ptr so that it will be deleted (and hidden) when the current function returns.
 
       dialog_progress->set_transient_for(*this);
       dialog_progress->show();
