@@ -28,22 +28,30 @@
 namespace Glom
 {
 
+/** Show the dialog, with the specified title.
+ * @param title The initial title. This will also be filled with the actual entered title.
+ * @param self_hosted This will be true if the user chose to self-host the database.
+ * @result This will be false if the user cancelled the dialog.
+ */
+bool show_dialog_new_database(Gtk::Window* parent_window, Glib::ustring& title, bool& self_hosted);
+
 class Dialog_NewDatabase : public Gtk::Dialog
 {
 public:
   Dialog_NewDatabase(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~Dialog_NewDatabase();
 
-  //Set sensible defaults:
+  ///Set sensible defaults:
   virtual void set_input(const Glib::ustring& title);
 
-  //Get the user input:
-  virtual void get_input(Glib::ustring& title);
+  ///Get the user input:
+  virtual void get_input(Glib::ustring& title, bool& self_hosted);
 
 protected:
   virtual void on_entry_title_changed();
 
   Gtk::Entry* m_entry_title;
+  Gtk::RadioButton* m_radiobutton_server_selfhosted;
   Gtk::Button* m_button_ok;
 };
 
