@@ -24,7 +24,9 @@
 #include <glom/libglom/data_structure/glomconversions.h>
 
 #include <glibmm/i18n.h>
+#include <gtkmm/messagedialog.h>
 #include <libgnome/gnome-url.h>
+#include <libgnome/gnome-help.h>
 #include <sstream> //For stringstream
 
 #include <iostream>
@@ -50,10 +52,13 @@ public:
   {
   }
 
-  predicate_UsesRelationshipHasRelationships(const sharedptr<const UsesRelationship> uses_relationship_name)
+  predicate_UsesRelationshipHasRelationships(const sharedptr<const UsesRelationship> uses_relationship_name, bool first_level_only = false)
   : m_relationship_name(uses_relationship_name->get_relationship_name()),
     m_related_relationship_name(uses_relationship_name->get_related_relationship_name())
   {
+    //If first_level_only, search for relationships that have the same top-level relationship, but have no related relationship.
+    if(first_level_only)
+      m_related_relationship_name = Glib::ustring();
   }
 
   virtual ~predicate_UsesRelationshipHasRelationships()
@@ -173,7 +178,7 @@ Glib::ustring Utils::string_replace(const Glib::ustring& src, const Glib::ustrin
 }
 
 
-Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const type_sort_clause& sort_clause)
+Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const Glib::ustring& extra_join, const type_sort_clause& sort_clause, const Glib::ustring& extra_group_by)
 {
   //TODO_Performance:
   type_vecConstLayoutFields constFieldsToGet;
@@ -182,10 +187,40 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
     constFieldsToGet.push_back(*iter);
   }
 
-  return build_sql_select_with_where_clause(table_name, constFieldsToGet, where_clause, sort_clause);
+  return build_sql_select_with_where_clause(table_name, constFieldsToGet, where_clause, extra_join, sort_clause, extra_group_by);
 }
 
-Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const type_sort_clause& sort_clause)
+
+static typedef std::list< sharedptr<const UsesRelationship> > type_list_relationships;
+
+static void add_to_relationships_list(type_list_relationships& list_relationships, const sharedptr<const LayoutItem_Field>& layout_item)
+{
+  if(!(layout_item->get_has_relationship_name()))
+    return;
+
+  //If this is a related relationship, add the first-level relationship too, so that the related relationship can be defined in terms of it:
+  type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item, true /* top_level_only */) );
+  if(iterFind == list_relationships.end()) //If the table is not yet in the list:
+  {
+    sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
+    uses_rel->set_relationship(layout_item->get_relationship());
+    list_relationships.push_front(uses_rel); //These need to be at the front, so that related relationships can use them later in the SQL statement.
+  }
+
+  //Add the relationship to the list:
+  iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item) );
+  if(iterFind == list_relationships.end()) //If the table is not yet in the list:
+  {
+    sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
+    uses_rel->set_relationship(layout_item->get_relationship());
+    uses_rel->set_related_relationship(layout_item->get_related_relationship());
+    list_relationships.push_back(uses_rel);
+  }
+
+ 
+}
+
+Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const Glib::ustring& extra_join, const type_sort_clause& sort_clause, const Glib::ustring& extra_group_by)
 {
   Glib::ustring result;
 
@@ -196,41 +231,16 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
   for(type_vecConstLayoutFields::const_iterator iter = fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
   {
     sharedptr<const LayoutItem_Field> layout_item = *iter;
-
-    if(layout_item->get_has_relationship_name())
-    {
-      //Add the relationship to the list:
-      type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item) );
-      if(iterFind == list_relationships.end()) //If the table is not yet in the list:
-      {
-        sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
-        uses_rel->set_relationship(layout_item->get_relationship());
-        uses_rel->set_related_relationship(layout_item->get_related_relationship());
-        list_relationships.push_back(uses_rel);
-      }
-    }
+    add_to_relationships_list(list_relationships, layout_item);
   }
 
   for(type_sort_clause::const_iterator iter = sort_clause.begin(); iter != sort_clause.end(); ++iter)
   {
     sharedptr<const LayoutItem_Field> layout_item = iter->first;
-
-    if(layout_item->get_has_relationship_name())
-    {
-      //Add the relationship to the list:
-      type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item) );
-      if(iterFind == list_relationships.end()) //If the table is not yet in the list:
-      {
-        sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
-        uses_rel->set_relationship(layout_item->get_relationship());
-        uses_rel->set_related_relationship(layout_item->get_related_relationship());
-        list_relationships.push_back(uses_rel);
-      }
-    }
+    add_to_relationships_list(list_relationships, layout_item);
   }
 
 
-
   Glib::ustring sql_part_fields;
   Glib::ustring sql_part_from;
 
@@ -303,6 +313,11 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
     }
   }
 
+  if(!extra_join.empty())
+  {
+    sql_part_leftouterjoin += (" " + extra_join + " ");
+  }
+
   if(!sql_part_from.empty())
     result += ("," + sql_part_from);
 
@@ -312,6 +327,12 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
   if(!where_clause.empty())
     result += " WHERE " + where_clause;
 
+  //Extra GROUP_BY clause for doubly-related records. This must be before the ORDER BY sort clause:
+  if(!extra_group_by.empty())
+  {
+    result += (" " + extra_group_by + " ");
+  }
+
   //Sort clause:
   if(!sort_clause.empty())
   {
@@ -754,4 +775,47 @@ Glib::ustring Utils::string_remove_suffix(const Glib::ustring& str, const Glib::
   return str;
 }
 
+/* Run dialog and response on Help if appropriate */
+
+int Utils::dialog_run_with_help(Gtk::Dialog* dialog, const Glib::ustring& id)
+{
+  int result = dialog->run();
+  while (result == Gtk::RESPONSE_HELP)
+  {
+    show_help(id);
+    result = dialog->run();
+  }
+  dialog->hide();
+  return result;
+}
+
+/*
+ * Help::show_help(const std::string& id)
+ *
+ * Launch a help browser with the glom help and load the given id if given
+ * If the help cannot be found an error dialog will be shown
+ */
+
+void Utils::show_help(const Glib::ustring& id)
+{
+  GError* err = 0;
+  const gchar* pId;
+  if (id.length())
+  {
+    pId = id.c_str();
+  }
+  else
+  {
+    pId = 0;
+   }   
+  if (!gnome_help_display("glom.xml", pId, &err))
+  {
+     std::string message = std::string(_("Could not display help: ")) + err->message;
+     Gtk::MessageDialog* dialog = new Gtk::MessageDialog(message, false, Gtk::MESSAGE_ERROR);
+     dialog->run();
+     delete dialog;
+     g_error_free(err);
+   }
+}
+
 } //namespace Glom
